__all__ = [
    "pause_for_frames",
    "create_sprite",
    "set_size",
    "set_position",
    "set_rotation",
    "move",
    "run_game",
    "run_animation",
    "get_x",
    "get_y",
    "is_key_pressed",
    "get_mouse_position",
    "is_mouse_clicked"
]

_pause_counter = 0

# Global variables for input state
_mouse_position = (0, 0)
_mouse_clicked = False
_key_states = {}

def pause_for_frames(n):
    global _pause_counter
    _pause_counter = n

_screen = None
_sprites = []

def _init_screen(sprites, width, height):
    global _screen, _sprites
    _sprites = sprites
    _screen = pygame.display.set_mode((width, height))
    pygame.display.set_caption("Scratch Environment")

def force_redraw():
    if _screen and _sprites:
        _update_input_state()
        _screen.fill((255, 255, 255))
        for sprite in _sprites:
            _screen.blit(sprite.image, sprite.rect)
        pygame.display.flip()
        pygame.time.delay(16)

import pygame

class Sprite:
    def __init__(self, image_path, x, y):
        self.image = pygame.image.load(image_path)
        self.original_image = self.image.copy()
        self.rect = self.image.get_rect()
        self.rect.topleft = (x, y)
        self.rotation = 0
        self.width_scale = 1
        self.height_scale = 1

    def set_size(self, width_scale, height_scale):
        self.width_scale = width_scale
        self.height_scale = height_scale
        self.update_image()

    def set_position(self, x, y):
        self.rect.topleft = (x, y)

    def set_rotation(self, angle):
        self.rotation = angle
        self.update_image()

    def update_image(self):
        image = pygame.transform.rotozoom(self.original_image, -self.rotation, 1)
        w, h = image.get_size()
        new_size = (int(w * self.width_scale), int(h * self.height_scale))
        self.image = pygame.transform.scale(image, new_size)
        self.rect = self.image.get_rect(center=self.rect.center)

    def move(self, dx, dy):
        self.rect.x += dx
        self.rect.y += dy

def create_sprite(image_path, x, y):
    return Sprite(image_path, x, y)

def set_size(sprite, width_scale, height_scale):
    sprite.set_size(width_scale, height_scale)

def set_position(sprite, x, y):
    sprite.set_position(x, y)

def set_rotation(sprite, angle):
    sprite.set_rotation(angle)

def move(sprite, dx, dy):
    sprite.move(dx, dy)

def run_game(sprites, width=600, height=600, on_loop=None):
    pygame.init()
    screen = pygame.display.set_mode((width, height))
    pygame.display.set_caption("Scratch Environment")
    clock = pygame.time.Clock()

    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

        global _pause_counter
        if _pause_counter > 0:
            _pause_counter -= 1
        else:
            if on_loop:
                on_loop()

        screen.fill((255, 255, 255))  # White background
        for sprite in sprites:
            screen.blit(sprite.image, sprite.rect)
        pygame.display.flip()
        clock.tick(60)

    pygame.quit()

def _update_input_state():
    """Internal function to update input state."""
    global _mouse_position, _mouse_clicked, _key_states
    
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            exit()
        elif event.type == pygame.MOUSEMOTION:
            _mouse_position = event.pos
        elif event.type == pygame.MOUSEBUTTONDOWN:
            _mouse_clicked = True
        elif event.type == pygame.MOUSEBUTTONUP:
            _mouse_clicked = False
        elif event.type == pygame.KEYDOWN:
            _key_states[event.key] = True
        elif event.type == pygame.KEYUP:
            _key_states[event.key] = False

def is_key_pressed(key):
    """
    Check if a specific key is pressed.
    Common keys: 'space', 'up', 'down', 'left', 'right', 'a', 'b', 'c', etc.
    """
    key_map = {
        'space': pygame.K_SPACE,
        'up': pygame.K_UP,
        'down': pygame.K_DOWN,
        'left': pygame.K_LEFT,
        'right': pygame.K_RIGHT,
        'a': pygame.K_a,
        'b': pygame.K_b,
        'c': pygame.K_c,
        # Add more keys as needed
    }
    
    if key.lower() in key_map:
        return _key_states.get(key_map[key.lower()], False)
    return False

def get_mouse_position():
    """Get the current mouse position as (x, y) coordinates."""
    return _mouse_position

def is_mouse_clicked():
    """Check if the mouse button was clicked."""
    return _mouse_clicked

def run_animation(sprites, animation_function, steps=10, width=600, height=600):
    global _screen, _sprites
    pygame.init()
    _init_screen(sprites, width, height)  # <--- This is key!
    clock = pygame.time.Clock()

    step = 0
    running = True
    while running and step < steps:
        _update_input_state()

        global _pause_counter
        if _pause_counter > 0:
            _pause_counter -= 1
        else:
            animation_function()
            step += 1

        force_redraw()
        clock.tick(60)

    pygame.quit()

def get_x(sprite):
    """Get the current x position of the sprite."""
    return sprite.rect.x

def get_y(sprite):
    """Get the current y position of the sprite."""
    return sprite.rect.y